- ![oop_polymorphie1.png](../assets/oop_polymorphie1_1700773974336_0.png)
  collapsed:: true
	- Grafik von Michael Kipp
- Es gibt zwei Probleme:
	- 1. Man könnte Instanzen von `Mover` erzeugen, obwohl dies nicht gewünscht ist.
	- 2. Die Methode `render()` der Klasse Mover ist leer und existiert nur zu dem Zweck, dass die Methoden der Unterklasse “bekannt gemacht” werden”.
- Um Problem Nr. 1 zu lösen, können wir `Mover` zu einer **abstrakten Klasse** machen. Dazu müssen wir nur das Wort `abstract` vor die Klassendefinition schreiben:
  
  
  ```java
  abstract class Mover {
  ...
  }
  ```
- Wenn du jetzt versuchst, ein neues Objekt vom Typ `Mover` herzustellen, bekommst du eine Fehlermeldung.
  
  ```java
  void setup() {
    Mover m = new Mover(); // FEHLER
  }
  ```
- Genau das wollten wir erreichen! Eine abstrakte Klasse kann ansonsten Instanzvariablen und Methoden haben, die sie weitervererbt. Abstrakte Klassen sind also ideal, um gemeinsame Funktionalität “nach oben” zu verschieben.
- Wie lösen wir Problem Nr. 2? Wir möchten einerseits Java signalisieren, dass alle Unterklasse die Methode `render()`haben, möchten aber andererseits keine “Dummy-Methode” einbauen.
  Zu diesem Zweck gibt es **abstrakte Methoden**. Eine abstrakte Methode hat keinen Code, sondern ist lediglich **ein Versprechen, dass alle (nicht-abstrakten) Unterklassen diese Methode implementieren**. Das ganze schreibt man - innerhalb der abstrakten Klasse - dann so:
  
  ```java
  abstract void render();
  ```
- Abstrakte Methoden können natürlich auch Parameter und Rückgabetyp definieren. Sie dürfen logischerweise nur in abstrakten Klassen vorkommen.
- Hier nochmal der komplette Code von `Mover`:
  
  ```java
  abstract class Mover {
    PVector location;
    PVector speed;
  
    Mover() {
      location =
      new PVector(random(0, width), random(0, height));
      speed =
      new PVector(random(-3, 3), random(-3, 3));
    }
  
    void update() {
      location.add(speed);
      if (location.x > width || location.x < 0) {
        speed.x = -speed.x;
      }
  
      if (location.y > height || location.y < 0) {
        speed.y = -speed.y;
      }
    }
  
    abstract void render();
  }
  ```
- Im Klassendiagramm schreibt man bei abstakten Klassen “<>” über den Klassennamen. Den Klassennamen schreibt man kursiv. Bei abstrakten Methoden schreibt man den Methodennamen kursiv.
- ![oop_abstrakteKlasse.png](../assets/oop_abstrakteKlasse_1700773816073_0.png)
- Unser Code ist jetzt gut strukturiert und robust.
- #+BEGIN_CAUTION
  Eine **abstrakte Klasse** verwendet man, um zu verhindern, dass Instanzen der Klasse erzeugt werden. Dazu stellt man das Schlüsselwort `abstract` vor die Klassendefinition.
  #+END_CAUTION
- Abstrakte Klassen können Instanzvariablen und Methoden enthalten und diese an die Unterklassen vererben.
- Soll eine Methode einer abstrakten Klasse keinen Code enthalten, sondern lediglich als Versprechen dienen, dass alle nicht-abstrakten Unterklassen diese Methode enthalten, dann kann man eine **abstrakte Methode** definieren. Auch die abstrakte Methode wird mit dem Schlüsselwort `abstract` markiert.
- Eine abstrakte Methode enthält keinen Code. Die Kopfzeile wird mit einem Semicolon abgeschlossen. Jede (nicht-abstrakte) Unterklasse muss diese Methode erfüllen, sonst kommt eine Fehlermeldung.
-
-
- ## Übung
- Hier gehts zu den [[Aufgaben Abstrakt]].
-
-
- ## Vor- und Nachteile
- #+BEGIN_PINNED
  **Aufgabe**
  Entscheide, was Vorteile bzw. was Risiken sind und ordnen sie zu:
  #+END_PINNED
	- Wiederverwendung von existierendem Code (Subklassen verwenden den Code der Superklassen)
	- zunehmende Unübersichtlichkeit in tief verschachtelten Vererbungshierarchien (In welcher Klasse wurde ein gewisses Attribut oder eine gewisse Methode definiert?)
	- einfachere Wartung (zusätzliche Attribute oder Methoden werden nur in der Superklasse ergänzt und stehen allen Subklassen zur Verfügung).
	- Ausnutzung der Polymorphie: Zur Laufzeit wird für dass gegebene Objekt entschieden, welche Implementierung einer Methode ausgeführt wird.
	- zu enge Kopplung zwischen den Klassen in der Hierarchie erschwert auch die Wartbarkeit (Änderungen haben Auswirkungen auf alle Subklassen, auch wenn dies gar nicht gewünscht ist. – Änderungen wirken nicht nur lokal sondern global aus.)
	- Vorteile:
		- ...
	- Nachteile:
		- ...