- ![](https://cdn.pixabay.com/photo/2016/12/09/18/30/database-schema-1895779_1280.png)
- Objekte stehen oft in **Abhängigkeiten** zueinander. Beispielsweise möchte ein Autohaus eine Kunden-Datenbank anlegen von verkauften Autos und ihren jeweiligen Besitzern. Zum Beispiel hier die Klasse Kunde:
- ```java
  class Kunde {
    private String name;
    private String adresse;
  
    public Kunde(String n, String a) {
      this.name = n;
      this.adresse = a;
    }
  }
  ```
- Die Klasse Auto soll neben Marke und Farbe auch den **Besitzer** enthalten. Jetzt könnten wir den Namen als String verwenden, aber das wäre umständlich, wenn ich vom Auto ausgehend alle Daten des Benutzers finden möchte. Stattdessen speichere ich einfach das dazugehörige **Kundenobjekt**.
- ```java
  class Auto {
    private String marke;
    private String farbe;
    private Kunde besitzer;
  
    Auto(String m, String f, Kunde k) {
      this.marke = m;
      this.farbe = f;
      this.besitzer = k;
    }
  }
  ```
- Der Vorteil, hier das Kundenobjekt zu speichern und nicht nur den Namen als String ist folgender:
  Man kann jetzt auf alle Informationen des Objekts zugreifen - hier z.B. die Adresse - ohne dass man das entsprechende Objekt vorher suchen muss.
  Man kann jetzt eine Beschreibung hinzufügen, in der man auch die Adresse des Kunden verwende:
- ```java
  class Auto {
  
    ...
  
    public void beschreibe() {
      println(this.besitzer.name + " aus " + this.besitzer.adresse +
      " hat einen " + this.marke + " in " + this.farbe);
    }
  }
  ```
- ### Beispielanwendung
- ```java
  void setup() {
    // Kunden erstellen
    Kunde barack = new Kunde("Barack Obama", "Amerika");
    Kunde angie = new Kunde("Angela Merkel", "Deutschland");
  
    // Autos erstellen
    Auto bmw = new Auto("BMW", "rot", angie);
    Auto porsche = new Auto("Porsche", "schwarz", barack);
    Auto rover = new Auto("Land Rover", "schwarz", barack);
  
    // Autos ausgeben
    bmw.beschreibe();
    porsche.beschreibe();
    rover.beschreibe();
  }
  ```
- #+BEGIN_NOTE
  **Aufgabe**
  Prüfe die Ausgabe des Quellcodes.
  #+END_NOTE
- Im Grunde hast du hier nichts neues gelernt. Du wusstest bereits, dass Variablen Objekte “enthalten” können (besser gesagt: sie zeigen auf sie). Jetzt hast du gesehen, dass auch Instanzvariablen auf Objekte zeigen können.
  Im Diagramm könnte man die Situation wie folgt darstellen:
- ![abhangigkeit_auto_besitzer.png](../assets/abhangigkeit_auto_besitzer_1700770927121_0.png)
	- Grafik von Michael Kipp
- Es ist immer wichtig, sich klar zu machen, dass die **Variablen** lediglich **Zeiger auf Objekte** sind. Wenn du z.B. aus einem Auto-Objekt heraus ein Kundenobjekt manipulierst, dann gilt diese Änderung natürlich “überall”, also auch, wenn du später auf den Kunden über die Variable `angie` zugreifst.
-
- ## Beispiel Ball
- Wir schauen uns unser Grafikbeispiel Ball aus den vorherigen Kapiteln an. Dabei ist in etwa folgender Code entstanden:
- ```java
  class Ball {
   private PVector pos;
   private PVector speed;
   public Ball() {
    pos = new PVector(random(5, width-5), random(5, height-5));
    speed = new PVector(random(-2, 2), random(-2,2));
   }
   
   public void render() {
    noStroke();
    ellipse(pos.x, pos.y, 10, 10);
   }
   
   public void update() {
    pos.add(speed);
    if (pos.x < 5 || pos.x > width-5) {
      speed.x = -speed.x;
    }
    if (pos.y < 5 || pos.y > height-5) {
      speed.y = -speed.y;
    }
   }
  }
  ```
- Wir animieren im Hauptprogramm drei Ball-Objekte:
- ```java
  Ball b1 = new Ball();
  Ball b2 = new Ball();
  Ball b3 = new Ball();
  
  void draw() {
    background(0);
    b1.render();
    b2.render();
    b3.render();
    b1.update();
    b2.update();
    b3.update();
  }
  ```
- Jetzt möchten wir **Verbindungen** zwischen je zwei Bällen hinzufügen. Jede Verbindung kann man sich als Objekt vorstellen, wo jeweils die zwei “Enden” gespeichert sind. Wir nennen die dazu notwendige Klasse `Link`.
- ```java
  class Link {
  private Ball ball1;
  private Ball ball2;
  
   public Link(Ball b1, Ball b2) {
    ball1 = b1;
    ball2 = b2;
   }
  
   public void render() {
    stroke(255);
    line(ball1.pos.x, ball1.pos.y, ball2.pos.x, ball2.pos.y);
   }
  }
  ```
- In unserem Hauptprogramm bauen wir zwei Links ein:
- ```java
  Ball b1 = new Ball();
  Ball b2 = new Ball();
  Ball b3 = new Ball();
  Link link1 = new Link(b1, b2);
  Link link2 = new Link(b2, b3);
  
  void draw() {
    background(0);
    link1.render();
    link2.render();
    b1.render();
    b2.render();
    b3.render();
    b1.update();
    b2.update();
    b3.update();
  }
  ```
- Ein Objekt kann also auch eine Beziehung zwischen zwei oder mehr Objekten sein. Dass Objekte aufeinander verweisen (= zeigen), ist in der objektorientierten Programmierung ganz normal.
-
-
- ### Beziehungstypen
	- Letztlich kann man die Beziehungen von Objekten auf 3 verschiedene Typen kategorisieren. Alle Typen sind aus den Datenbanken eigentlich schon bekannt.
	- [[1:1]]-Beziehung
	  Einer Person gehört genau ein Personalausweis.
	  Ein Personalausweis gehört genau zu einer Person.
	  Eine Person kann genau mit einer anderen Person verheiratet sein.
	- [[1:n]]-Beziehung
	  Eine Klasse besteht aus mehreren Schülerinnen und Schülern.
	  Eine Schülerinnen oder Schüler gehört zu einer Klasse.
	- [[n:m]]-Beziehung
	  Ein Lehrender hat mit vielen Lernenden zu tun.
	  Ein Lernender hat mit vielen Lehrenden zu tun.
- Für die technische Umsetzung klicke auf die Beziehungstypen.
- #+BEGIN_CAUTION
  **Grundsatz**: Das Objekt, welches Dienste eines anderen Objektes nutzen will, muss eine Referenz (Zeiger) auf das andere Objekt besitzen.
  #+END_CAUTION
-
-
- ### Aufgaben
- Hier gehts zu den [[Aufgaben Beziehungen]]
-