- ![data-protection.jpg](../assets/data-protection_1700772051214_0.jpg)
- Software-Objekte (als Programmeinheiten) werden so konzipiert, dass Details über den inneren Aufbau verborgen werden und Änderungen von Objektzuständen nur über dafür vorgesehene Methoden erfolgen können.
- #+BEGIN_NOTE
  Das Verbergen des inneren Aufbaus wird realisiert, indem man keinen direkten Zugriff auf die Attribute zur Verwaltung der internen Daten eines Objekts ermöglicht. Man nennt diese Vorgehensweise auch **Datenkapselung**.
  #+END_NOTE
- Man geht also auch hier nach dem **Geheimnisprinzip** vor. Software-Objekte sollen - genau wie Automotoren - von Benutzern nur in der vorgesehenen Weise genutzt werden, keinesfalls soll die interne Struktur durch Benutzer verändert werden. Das hat den Vorteil, dass ein Objekt als Software-Baustein genau das Verhalten hat, das von den Entwicklern des Bausteins realisiert wurde.
-
- ## Zugriffsrechte
- #+BEGIN_NOTE
  Um interne Daten kapseln zu können, werden **Zugriffrechte** festgelegt. Der Entwickler einer Klasse hat die Möglichkeit, Attribute und Methoden einer Klasse als **öffentlich** oder **privat** zu deklarieren. Lesende und schreibende Zugriffe auf Attribute bzw. Methoden eines Objekts sind nur möglich, wenn diese öffentlich sind. Private Attribute bzw. Methoden können dagegen nur bei der Implementierung der betreffenden Klasse benutzt werden.
  #+END_NOTE
- ### Vorteile
	- Veränderungen ohne Einschränkungen in Zusammenarbeit
	- Öffentliche Schnittstelle erhöht Übersichtlichkeit
	- Reduktion unerwünschter Interaktionen zwischen Programmteilen
	- Fremdnutzer haben es einfacher, da nur öffentliche Schnittstellen betrachtet werden müssen
	- Objekte kontrollieren die Konsistenz ihrer Daten selbst (stabile Software)
	- Implementierung kann intern geändert werden, solange der Methodenname unverändert bleibt, kann das Objekt wie gewohnt benutzt werden (lokale Änderung ohne globale Auswirkung)
- ### Nachteile
	- gegebenenfalls Geschwindigkeitseinbußen
	- Zugriffsfunktion ist zusätzlicher Programmieraufwand
-
- ## Kennzeichnung
- public (`+`): Funktionen aller Klassen erhalten Zugriff
  collapsed:: true
	- #+BEGIN_NOTE
	  Wenn eine Variable oder Methode public deklariert ist, dann ist sie von jeder Klasse aus siehtbar und bearbeitbar, die eine Referenz(ein Objekt) zu der Klasse der Variable/Methode besitzt.
	  #+END_NOTE
- protected (`#`): Datenzugriff nur für abgeleitete Klassen
  collapsed:: true
	- #+BEGIN_NOTE
	  Wenn eine Variable oder Methode protected deklariert ist, dann kann nur vom gleichen Packet, oder von Klassen die von der Klasse der Variable/Methode erben, aus darauf zugegriffen werden. Was erben bedeutet, bzw. Vererbung ist, erkläre ich in einem späteren Tutorial.
	  #+END_NOTE
- privat (`-`): Datenzugriff nur von Klassenmethoden selbst
  collapsed:: true
	- #+BEGIN_NOTE
	  Wenn eine Variable oder Methode private deklariert ist, dann ist sie nur in der Klasse in der sie erstellt wurde sichtbar, also nicht von anderen Klassen.
	  #+END_NOTE
- default
  collapsed:: true
	- #+BEGIN_NOTE
	  Wenn eine Variable oder Methode default deklariert ist, also keinen Zugrifsmodifikator hat, dann wird sie vom Packet in dem sie befindlich ist gesehen und kann via Referenz auf das Objekt der Klasse verändert/gesehen werden.
	  #+END_NOTE
-
- ## Erklärung am Beispiel
- Quellcode
  
  ```java
  public class ClassA {
     public void ausgabe(){
       ClassB1 object = new ClassB1();
       object.var1 = 19;
       println(object.var1);
       object.methode();
     }
   }
  
  public class ClassB1 {
    public int var1 = 3; 
  
    public void methode(){
      println("Ich bin eine Methode.");
    }
  }
  
  public class ClassB2 {
    private int var1 = 3; 
  
    private void methode(){
      println("Ich bin eine Methode.");
    }
  }
  ```
  
  Wird nun Klasse B1 genutzt, steht einer Verwendung der Variablen `var1` nichts im Wege, da die Variable auf **public** gesetzt wurde.
  
  #+BEGIN_WARNING
  Wird jedoch Klasse B2 genutzt, kann die Variable var1
  
  1. nicht verändert werden (Zeile 4)
  2. nicht einfach ausgelesen (Zeile 5)
  
  da sie auf private gesetzt ist.
  
  Analog kann bei Verwendung von ClassB2 die Methode methode() nicht verwendet werden (Zeile 6).
  #+END_WARNING
- ### Doch was, wenn man auf eine Variable einer anderen Klasse zugreifen muss?
- #+BEGIN_NOTE
  Dafür werden so genannte get- bzw. set-Methoden geschrieben. Diese sind public gesetzt und können so von außen genutzt werden. Jedoch kann eine Fehlerabfrage innerhalb der set-Methode das Attribut schützen.
  #+END_NOTE
- **Beispiel**
	- Wenn ich zulassen möchte, dass der Radius meines Balles verändert wird, kann eine **set**-Methode entwickelt werden, welche auf das eigene Attribut zugreift. Der Vorteil solcher Methoden sind Regulierungen/Kontrollen. Beispielweise kann in der `setRadius(int neuerRadius)`-Methode ausgeschlossen werden, dass der Ball größer als 50 px wird.
-
-
-
-
- ### Zusammenfassung
- #+BEGIN_TIP
  Alle Attribute haben den Zugriff **private**.
  Die Methoden sind **public**.
  
  *Hinweis: Standardmäßig ist in Processing alles public.*
  
  Das Schlüsselwort private würde dann dafür sorgen, dass ein direkter Zugriff wie sammler.name nicht erlaubt ist, sondern nur über Funktionen der Klasse ermöglicht wird. In Processing ist das Prinzip der Datenkapselung nicht umgesetzt. Dies hat technische Gründe.
  #+END_TIP