## Zahlen
- | Typ | Beschreibung | Wertebereich |
  | ---- | ---- | ---- |
  | `byte` | sehr kleine Zahl (8 Bit) | -128 bis 127 |
  | `int` | große Zahlen (32 Bit) | -2147483648 bis 2147483647 |
  | `float` | Zahl mit Nachkommastelle (32 Bit) | z.B. -0.5 ; 0 ; 109.333 |
  | `double` | Zahl mit mehr Nachkommastellen (64 Bit) | 15 bis 16 Stellen |
- ### Umwandlung zwischen int und float (Typcasting)
  
  ```java
  int x = (int) (3.9*2);
  ```
-
-
- ## Wahrheitswerte
- | Typ | Beschreibung | Wertebereich |
  | ---- | ---- | ---- |
  | `boolean` | Wahrheitswert | `false` und `true` |
-
- ## Zeichen
- | Typ | Beschreibung | Wertebereich |
  | ---- | ---- | ---- |
  | `char` | einzelnes Zeichen | z.B. `a` oder `%` |
- Du kannst ein einzelnes Zeichen in einer Variable abspeichern. Das Zeichen wird immer mit einfachen Anführungszeichen umgeben:
  
  ```java
  char c1 = 'a';
  char c2 = '5';
  char c3 = '%';
  println(c1 + c2 + c3);
  ```
- Intern wird ein Zeichen als ganze Zahl repräsentiert. Diese Zahl entspricht einer standardisierten Tabelle. Für die handelsüblichen Buchstaben ist das zum Beispiel der **ASCII-Code** (ohne Umlaute). ASCII steht für “American Standard Code for Information Interchange” und ist ganz einfach eine Tabelle, wo für jedem Buchstaben/Zeichen eine Zahl zugeordnet wird. Heutzutage verwendet man das Unicode-System, welches für fast alle Sprachen der Welt entsprechende Buchstaben-/Zeicheneinträge hat. Eine konkrete Enkodierung von Unicode ist z.B. **UTF-8**.
-
- ## PVector
- Dieser Datentyp ist für uns so wichtig, dass dafür eine eigene Informationsseite existiert.
- [[PVector]]
-
-
- ## Zeichenketten
- String ist englisch für “Schnur” oder “Kette”. Das deutet darauf hin, dass ein String eine Aneinanderreihung von Zeichen (engl. character) ist. Man sagt deshalb auch Zeichenkette. Der Typ String wird im Gegensatz zu int, float, char … groß geschrieben. Warum das so ist, erfährst du, sobald du Klassen und Objekte kennengelernt hast.
- In String-Variablen können Texte ablegt werden. Der Text wird mit doppelten Anführungszeichen gekennzeichnet:
  
  ```java
  String foo = "Dies ist ein Text!";
  ```
- Du kannst Strings zusammenkleben, dies nennt man **String-Konkatenation** und funktioniert mit dem normalen Plus-Zeichen:
- ```java
  String message = "hallo";
  println(message + " bob");
  ```
-
- ### Methoden
- Du kennst bisher die eigentliche Kurzform, die natürlich verwendet werden kann:
  
  ```java
  String name = "Harry";
  ```
- Da Strings eine eigene Klasse sind, ist die formale Definition:
  
  ```java
  String name = new String("Harry");
  ```
- In der Klasse sind jedoch verschiedene - sehr nützliche - Methoden definiert:
	- `equals()`: testet, ob der angegebene String inhaltsgleich mit dem String ist (gibt true oder false zurück)`
	- `length()`: gibt die Länge des Strings zurück
	- `contains()`: testet, ob der angegebene String im eigenen String enthalten ist (gibt true oder false zurück)
	- `indexOf()`: einen String innerhalb eines anderen Strings suchen (gibt Position des gefundenen Strings oder -1 zurück)
	- `toLowerCase()`: gibt kleingeschriebene Variante zurück
	- `toUpperCase()`: gibt großgeschriebene Variante zurück
	- `substring()`: einen Teilstring herstellen - es wird der Startbuchstabe (inklusive) und der Endbuchstabe (exklusive) als Indexzahl angegeben (daran denken, dass der erste Buchstabe Index 0 hat!)
- #+BEGIN_CAUTION
  Wichtig ist, dass **keine der Methoden den String, auf dem die Methode aufgerufen wird, verändert**. Tatsächlich ist es so, dass Strings nie im Verlauf ihres Lebens verändert werden können. String-Objekte sind sogenannte **Immutables**. In der Praxis heißt das, dass Methoden wie toLowerCase() einen neuen String zurückgeben, der alte String aber erhalten bleibt.
  String-Objekte können nie verändert werden. Bei Methodenanwendung wird stattdessen ein neuer String erzeugt.
  #+END_CAUTION
- ### Identität und Gleichzeit
- Wir haben gesagt, dass man bei Strings immer die Methode equals() verwenden sollte, um zu testen, ob zwei Strings gleich sind.
- Dies liegt daran, dass auch Strings Objekte sind und Stringvariablen auf diese Objekte zeigen. Wenn du Strings mit == vergleichst, kann es also sein, dass du ein false bekommst, obwohl die Strings “gleich” sind:
  
  ```java
  String s = "hallo";
  String s2 = new String("hallo");
  
  println(s == s2);
  ```
- Ergebnis: **false**
-
- ## Übung
- ### Aufgabe kleine Wanze
  collapsed:: true
	- #+BEGIN_PINNED
	  Schreibe ein Programm, das das Wort “Wanze” darstellt. Bei jedem Mausdruck, wird das Wort um einen Buchstaben kürzer. Also erst “Wanze”, dann “anze”, dann “nze” und so weiter.
	  Sobald das Wort ganz verschwunden ist, erscheint wieder “Wanze” beim nächsten Mausdruck und das ganze beginnt von vorn.
	  #+END_PINNED
- ### Aufgabe Palindromtester
  collapsed:: true
	- #+BEGIN_PINNED
	  Entwickle einen Algorithmus, der testet, ob ein Wort ein Palindrom ist.
	  #+END_PINNED
	- #+BEGIN_TIP
	  Palindrome sind Wörter, die vorwärts wie rückwärts die selbe Bedeutung haben:
	  ANNA, OTTO, LAGERREGAL, RENTNER
	  #+END_TIP
- ### Aufgabe Quersumme
	- #+BEGIN_PINNED
	  Entwickle ein Programm, welches zu einer gegebenen Zahl die Quersumme bildet.
	  #+END_PINNED
- ### Aufgabe Tastatur-Parser
  collapsed:: true
	- #+BEGIN_PINNED
	  Die klassische Eingabe mittels Tastatur ist in Processing nicht vorgesehen. Aber du kannst Processing dahingehend erweitern. Schreibe einen Algorithmus, der die Tastatureingabe ausliest und in der Konsole ausgibt.
	  #+END_PINNED
	- #+BEGIN_PINNED
	  Erweitere dein Programm, um eine Ausgabe auf der Grafikoberfläche.
	  Erweitere deinen Algorithmus so, dass auch das Löschen von Zeichen realisiert werden kann.
	  #+END_PINNED
- ### Aufgabe Passwortgenerator (schwer)
  collapsed:: true
	- #+BEGIN_PINNED
	  Der Nutzer soll einen Satz eingeben können. Die ersten Buchstaben eines jeweiligen Wortes werden verwendet, um ein Passwort zu generieren. Gib das Passwort aus.
	  #+END_PINNED
	  
	  *Beispiel: Aus “Heute scheint die Sonne stark.” folgt “HsdSs”*