- Das interessanteste an Objekten ist ihre **Wiederverwendbarkeit**. Diese Wiederverwendbarkeit hat dazu geführt, dass eine Sprache wie Java (Processing) erweitert wird, indem eine Reihe von Klassen von vornerein bereit gestellt wird. Sie kennen z.B. die Klasse String, die bei Java von Haus aus zur Verfügung steht. Eine Kollektion von Klassen nennt man auch **Bibliothek**. Die in Java “eingebaute” Bibliothek nennt man die **Java-Standardbibliothek**.
-
- ## Klasse definieren
-
- Um eine Klasse zu definieren, erzeugen wir einen neuen Reiter (Tab) mit dem Namen der Klasse (z.B. `Ball`).
- ![klasse_definieren_grafik1.png](../assets/klasse_definieren_grafik1_1700769818333_0.png)
  collapsed:: true
	- Grafik von Michael Kipp
-
- #+BEGIN_TIP
  Dabei wird der Klassenname stets groß geschrieben werden.
  #+END_TIP
- Um eine Klasse Ball mit drei Eigenschaften zu definieren, schreiben Sie:
- ```Java
  class Ball {
     PVector pos;
     int d; // Durchmesser
  }
  ```
-
- Die Eigenschaften nennt man auch **Instanzvariablen**, da es Variablen sind, die immer jeweils einem Objekt (einer Instanz) gehören.
- ![klasse_definieren_grafik2.PNG](../assets/klasse_definieren_grafik2_1700769718053_0.PNG)
-
- ## Objekte erzeugen
- Sobald Sie an anderer Stelle im Code eine Instanz einer Klasse benötigen, müssen Sie zunächst Variablen deklarieren, die eine solche Instanz speichern können:
- ```javascript
  Ball ball;
  Ball foo;
  Ball einBall;
  ```
- Die Klasse `Ball`ist gleichzeitig ein **Datentyp** (genauso wie int oder String). Man sagt daher:
  “Variable foo ist vom Typ Ball.”
- Jetzt können Sie neue Instanzen erzeugen und jeweils in den Variablen ablegen:
- ```Java
  ball = new Ball();
  foo = new Ball();
  einBall = new Ball();
  ```
- In dem Beispiel werden also drei Ball-Objekte erzeugt. Sie können das natürlich auch jeweils auf einer Zeile ausführen:
- ```Java
  Ball ball = new Ball();
  Ball foo = new Ball();
  Ball einBall = new Ball();
  ```
- Der Prozess des Instanziierens führt dazu, dass ein neues Objekt (eine Instanz) erschaffen wird. Wir stellen uns das als Häuschen in Object City vor. Jedes Objekt hat konkrete Werte für die Eigenschaften xpos, ypos und d.
- Die Variablen *ball*, *foo* und *einBall* enthalten nicht etwa diese Häuschen, sondern nur die Adresse des Häuschens. Man sagt auch, dass die Variable auf das Objekt zeigt.
- ![klasse_definieren_grafik3.PNG](../assets/klasse_definieren_grafik3_1700769739258_0.PNG)
- #+BEGIN_NOTE
  **Aufgabe**
  Versuche folgende Befehle:
  Ball ball = new Ball();
  println(ball);
  Was erscheint in der Konsole?
  #+END_NOTE
	- *Lösung*
	  ...
-
- ## Variablenzugriff per Punktnotation
- Um auf die Eigenschaften zuzugreifen, bedienen wir uns der Punktnotation. Hier setzen wir erstmal die Eigenschaften der drei Objekte von oben auf bestimmte Werte:
- ```Java
  ball.pos.x = 100;
  ball.pos.y = 100;
  ball.d = 40;
  foo.pos.x = 50;
  foo.pos.y = 30;
  foo.d = 20;
  einBall.pos.x = 20;
  einBall.pos.y = 10;
  einBall.d = 20;
  ```
- Processing/Java benötigt die Punktnotation, um dasjenige Objekt (Häuschen) zu finden, bei dem die Änderung vorgenommen werden soll:
- ![klasse_definieren_grafik4.PNG](../assets/klasse_definieren_grafik4_1700769751795_0.PNG)
- Sofern man die Werte eines Objekts das erste Mal setzt, nennt man das auch ein Objekt initialisieren. Die Eigenschaften eines Objekts können sich natürlich während des Programmablaufs ändern, z.B. die Variablen xpos und ypos in dem Beispiel, sofern die Objekte animiert werden sollen. Mit dem Begriff Zustand meint man die aktuelle Belegung aller Eigenschaften (Instanzvariablen) eines Objekts. Der Zustand kann sich also fortwährend ändern.
-