## Theorie
- Objekte können auch Aktionen beinhalten. Ein Mensch kann sich z.B. fortbewegen oder etwas sagen. Zu beachten ist, dass sich nur ein konkreter Mensch (Objekt) fortbewegen kann, nicht die ganze Klasse! In Processing kann man Funktionen in Klassen einbetten. Solche Funktionen werden oft Methoden genannt.
- ## Man unterscheidet in folgende Methoden
	- | sondierende Methode | manipulierende Methode | Konstruktor-Methode |
	  | ---- | ---- | ---- |
	  | auch Funktion/ Anfrage genannt | auch Prozedur / Aufruf genannt |  |
	  | liest Attributwerte oder berechnet etwas | verändert Attributwerte | erzeugt Objekt einer Klasse |
	  | haben einen Rückgabewert | hat keinen Rückgabewert | weist Attributen Startwerte zu und hat den selben Bezeichner wie die Klasse nur mit () |
	- #+BEGIN_TIP
	  Genau genommen gibt es in Java nur Funktionen...mit Rückgabewert oder "leerem Rückgabewert" (void).
	  #+END_TIP
-
- ## Methoden definieren
- Selbst definierte Methoden erhalten bei Funktionen den Datentyp des Rückgabewerts als Kennwert oder das Schlüsselwort `void`, falls es sich bei der Methode um eine Prozedur handelt. w> Schreibe Methodennamen immer klein!
- ```java
  class Ball {
     PVector pos;
     int durchmesser;
  
     void zeichne() {
        ellipse(pos.x, pos.y, durchmesser, durchmesser);
     }
  
     void bewege() {
        pos.x = pos.x + 1;
     }
  }
  ```
-
- Methoden werden genauso wie Funktionen definiert, d.h. sie haben einen Namen, mehrere (oder keine) Parameter und einen Rückgabetyp.Sie müssen innerhalb der geschweiften Klammern der Klassendefinition definiert werden. Innhalb der Methoden kann man auf alle Instanzvariablen der Klasse zugreifen (im Beispiel: xpos, ypos, durchmesser).
-
- Hier ein Beispiel einer Methode mit Parametern:
  
  ```java
  class Ball {
     ...
  
     void setPosition(int xp, int yp) {
       pos.x = xp;
       pos.y = yp;
     }
  
  }
  ```
- Hier ein Beispiel einer Methode mit Rückgabe:
  
  ```java
  class Ball
  {
     ...
  
     float distToMouse() {
        return dist(mouseX, mouseY, pos.x, pos.y);
     }
  
  }
  ```
-
- ## Methoden verwenden
- Um eine Methode aufzurufen, bedient man sich ebenfalls der Punktnotation:
  
  ```java
  Ball ball = new Ball();
  
  ball.pos.x = 100;
  ball.pos.y = 100;
  ball.durchmesser = 40;
  
  ball.zeichne(); // Methode zeichne() dieses Objekts aufrufen
  ```
-
-
- ## Konstruktor
- Sobald man `new Ball()` aufruft, wird eine spezielle Methode der Klasse aufgerufen: der **Konstruktor**. Ein Konstruktor ist wie eine Methode und wird genau dann aufgerufen, wenn das Objekt erzeugt wird. Er wird häufig verwendet, um die Eigenschaften zu setzen (**Initialisierung**).
  Der sogenannte **leere Konstruktor** wird automatisch erzeugt, wenn Sie keine eigenen Konstruktoren erzeugen. Er enthält aber keine besondere Funktionalität.
-
- ### Einfacher Konstruktor
  Man kann selbst einen Konstruktor schreiben, um z.B. Eigenschaften auf einen Anfangswert zu setzen:
- ```java
  class Ball
  {
    PVector pos;
    int durchmesser;
  
     // eigener Konstruktor
     Ball()
     {
        pos.x = 50;
        pos.y = 50;
        durchmesser = 20;
     }
  
     // hier kommen die weiteren Methoden
  }
  ```
-
- Ein Konstruktor sieht aus wie eine Methode, **heißt aber immer genauso wie die Klasse**, wird also (anders als Methoden) groß geschrieben.
- Man kann im Konstruktor beliebigen Code unterbringen. Zum Beispiel kann man eine kurze Ausgabe einbauen, die signalisiert, dass der Konstruktor aufgerufen wurde.
  
  ```java
  class Ball
  {
    PVector pos;
    int durchmesser;
  
     // eigener Konstruktor
     Ball()
     {
        pos.x = 50;
        pos.y = 50;
        durchmesser = 20;
        println("Neuen Ball erzeugt.");
     }
  
     // hier kommen die weitere Methoden
  }
  ```
- #+BEGIN_PINNED
  Kopiere den vorangegangenen Quellcode und probiere folgenden Befehl aus:
  #+END_PINNED
  
  ```java
  void setup(){
    Ball b1 = new Ball();
    Ball b2 = new Ball();
  }
  ```
-
-
- ### Konstruktor mit Parameter
- Man kann dem Konstruktor auch Parameter mitgeben, wenn man z.B. den Durchmesser übergeben möchte:
  
  ```java
  class Ball
  {
    PVector pos;
    int durchmesser;
  
     // eigener Konstruktor
     Ball(int x, int y, int d)
     {
        pos.x = x;
        pos.y = y;
        durchmesser = d;
     }
  }
  ```
- Dieser Konstruktor wird bei der Instanziierung wie folgt aufgerufen:
  
  ```java
  void setup() {
    Ball ball = new Ball(70, 60, 30);
  }
  ```
- #+BEGIN_IMPORTANT
  WICHTIG
  Sobald du einen eigenen Konstruktor definierst, wird der leere Konstruktor nicht mehr automatisch erzeugt.
  #+END_IMPORTANT
- Das hat zur Folge, dass dieser Code jetzt fehlerhaft ist:
  
  ```java
  void setup() {
    Ball ball = new Ball(70, 60, 30);
    Ball ball2 = new Ball(); // FEHLER: Konstruktor fehlt!
  }
  ```
- Man kann auch mehrere Konstruktoren gleichzeitig definieren. Jeder dieser Konstruktoren ist dann bei der Instanziierung verwendbar:
  
  ```java
  class Ball
  {
    PVector pos;
    int durchmesser;
  
     // Konstruktor 1 für Faule
     Ball()
     {
        pos.x = 50;
        pos.y = 50;
        durchmesser = 20;
        println("Konstruktor 1 aufgerufen.");
     }
  
     // Konstruktor 2 für Raffinierte
     Ball(int x, int y, int d)
     {
        pos.x = x;
        pos.y = y;
        durchmesser = d;
        println("Konstruktor 2 aufgerufen mit " + x + ", " + y + ", " + d);
     }
  }
  ```
- Dann kann man folgendermaßen instanziieren:
  
  ```java
  void setup() {
    Ball ball = new Ball(70, 60, 30); // Konstruktor 2
    Ball foo = new Ball(); // Konstruktor 1
    Ball einBall = new Ball(); // Konstruktor 1
  }
  ```
-
-
- ## Übung
- https://cdn.pixabay.com/photo/2017/03/11/08/14/handheld-game-console-2134571_1280.png
- #+BEGIN_PINNED
  **Aufgabe**
  Entwickle eine eigene Klasse Pacman.
  #+END_PINNED
- #+BEGIN_PINNED
  **Aufgabe**
  Erzeuge einen Konstruktor mit geeigneten Attributen für deinen Pacman.
  #+END_PINNED
- #+BEGIN_PINNED
  **Aufgabe**
  Erzeuge innerhalb einer geeigneten Methode die Mundbewegung deines Pacman.
  #+END_PINNED
- #+BEGIN_PINNED
  **Aufgabe**
  Erweitere die Funktion um die Steuerbarkeit deiner Spielfigur.
  #+END_PINNED
- #+BEGIN_PINNED
  **Aufgabe**
  Finde eine geeignete Lösung für das Verhalten deiner Spielfigur am Rand deiner Spielfläche.
  #+END_PINNED
-
-
- ## Weitere Übungen
- Weitere Übungsaufgaben findest du hier [[Aufgaben Klassen und Methoden]].
-
-
- ## Kontrollfragen
- Erläutere die verschiedenen Arten von Methoden. #card
  collapsed:: true
	- Antwort:
	  Konstruktormethode - weist den Attributen Initialwerte zu
	  Funktionen (sondierende Methoden) - haben einen Rückgabewert, Kennzeichnung durch Angabe des Rückgabewertes und `return`
	  Prozeduren (manipulierende Methoden) - haben keinen Rückgabewert - Kennzeichnung durch Schlüsselwort `void`, denn es gibt einen "leeren Rückgabewert"