## Eindimensionale Felder
- Wenn eine Variable sowas wie ein Becher ist, dann ist ein Array sowas wie ein Becherhalter. Nehmen wir eine Integer-Variable für die x-Position eines Balls oder auch eine Float-Variable oder eine boolsche Variable.
- ```java
  int x;
  float foo = 42.3;
  boolean play = true;
  ```
- Die Variable `x` ist nur deklariert, wird aber automatisch mit dem Wert Null gefüllt.
- Jetzt möchten wir ein **Array** anlegen, der z.B. gleich 5 x-Positionen speichern kann. Das machen wir in zwei Schritten. Zunächst deklarieren wir eine Variable, die einen Becherhalter enthalten kann. Man beachte, dass der Becherhalter an sich noch nicht existiert!
- ```java
  int[] xarray; //Arrayvariable deklarieren
  ```
- Ein Array ist ein **Objekt**. Die Variable `xarray` kann einen Array speichern, ist aber jetzt noch leer. Wie wir wissen, jetzt Processing/Java daher eine Markierung namens `null` ein.
- Jetzt müssen wir noch den Becherhalter erzeugen. Hier müssen wir uns festlegen, wie viele Becher genau das Array halten kann. Stellen Sie sich das Array wie ein Muffin-Backblech vor, da müssen Sie sich bei Herstellung auch entscheiden, wieviele Muffin-Löcher Sie reinstanzen.
- Wir nehmen mal 5 Becher:
  
  ```java
  xarray = new int[5]; //Array mit 5 Elementen erzeugen
  ```
- Häufig macht man beide Schritte in einem:
  
  ```java
  int[] xarray = new int[5];
  ```
- Unseren Becherhalter kann man sich wie ein Gitter vorstellen, eine Batterie von Behältern. Die Variable `xarray` zeigt jetzt auf dieses Gebilde.
- Zu beachten ist, dass der Datentyp dieser Variable `int[]` ist, also ein Integer-Array.
-
- ## Zugriff auf ein Array
  collapsed:: true
	- Jetzt hat man plötzlich 5 Becher statt nur einen. Wir nennen die Becher auch Array-Elemente. Wie kommt man also an ein Element ran? Ganz einfach: die Elemente sind durchnummeriert, und zwar von **0 bis 4**. Diese Zahlen nennt man auch **Indices** bzw. im Singular **Index**.
	- Beispiel
	  
	  ```java
	  println(xarray[0]);
	  ```
-
- ## Werte zuweisen
  collapsed:: true
	- Dies stellt sich nicht viel komplizierter dar:
	  
	  ```java
	  xarray[3] = 42;
	  ```
	- Wenn du genau hinschaust, wirst du feststellen, dass sich ein einzelnes Element wie xarray[0] ganz genau so verhält wie eine handelsübliche Integer-Variable. Man muss sich nur an diese eckigen Klammern gewöhnen.
	- Du kannst z.B. auch mit Array-Elementen rechnen:
	  
	  ```java
	  int result = xarray[0] + xarray[5] - 10;
	  ```
-
- ## Größe eines Arrays
  collapsed:: true
	- Die Länge eines Arrays kann man mit Hilfe der Funktion `xarray.length` abrufen.
	- **Beispiel**
	  
	  ```java
	  int[] xarray = new int[555];
	  println(xarray.length);
	  ```
	- ### Beliebte Fehler
		- Der beliebteste Anfängerfehler bei Arrays ist es, nicht zu beachten, dass Informatiker immer bei Null anfangen zu zählen. Ist ja auch merkwürdig. Versuche mal:
		  
		  ```java
		  int[] xarray = new int[10];
		  xarray[10] = 23; // Index 10 existiert nicht!
		  ```
		- #+BEGIN_WARNING
		  Processing sagt zu recht: ArrayIndexOutOfBoundsException. Eine ‘Exception’ (engl. Ausnahme) bedeutet beim Programmieren ‘Fehler’. In diesem Fall ist der Fehler, dass das Element 10 nicht existiert. Es existieren nur Elemente mit Namen 0 bis 9.
		  #+END_WARNING
-
- ## Kurzschreibweise: Array erzeugen und befüllen
  collapsed:: true
	- Wenn du schnell ein Array erzeugen und gleich mit konkreten Werten befüllen möchtest, schreibst du:
	  
	  ```java
	  int[] zahlen = {8, 42, -3, 200};
	  ```
	- Dieses Konstrukt erzeugt eine neue Array-Variable `zahlen` der Länge 4 mit den entsprechenden Werten.
	- #+BEGIN_WARNING
	  Vorsicht: Die Kurzschreibweise darfst du nur dann verwenden, wenn die Variable deklariert wird, also ganz am Anfang der Lebensdauer einer Array-Variablen. Die Kurzschreibweise ist später nicht mehr erlaubt.
	  #+END_WARNING
	- Das hier geht nicht:
	  
	  ```java
	  int[] zahlen = new int[3];
	  zahlen = { 5, 20, 8 }; // FEHLER! Nur bei Deklaration erlaubt.
	  ```
-
-
- ## Array ausgeben
  collapsed:: true
	- In Processing kannst du übrigens mit
	  
	  ```java
	  println(zahlen);
	  ```
	  sehr bequem die Inhalte eines Arrays betrachten - probiere es aus.
-
-
- ## Übung
- ### Aufgabe Fibonacci-Zahlen
  collapsed:: true
	- #+BEGIN_PINNED
	  **Aufgabe - Fibonacci-Zahlen**
	  Befülle ein Array der Länge 15 mit den Fibonacci-Zahlen. Nutze dabei die Eigenschaften eines Arrays.
	  #+END_PINNED
- ### Aufgabe Summe
  collapsed:: true
	- #+BEGIN_PINNED
	  **Aufgabe - Summe**
	  Berechne die Summe von allen Elementen des Array und gib diese auf der Konsole aus.
	  #+END_PINNED
- ### Aufgabe mehrere Arrays
  collapsed:: true
	- #+BEGIN_PINNED
	  **Aufgabe - mehrere Arrays**
	  Gegeben sind die Arrays a,b und c. In Array c sollen die Summe der beiden Arrays a und b im jeweiligen Indice berechnet werden.
	  #+END_PINNED
- ### Aufgabe Arrays zusammenführen
  collapsed:: true
	- #+BEGIN_PINNED
	  **Aufgabe - Array zusammenführen**
	  Gegeben sind zwei Array a und b. Erzeuge ein Array c, der so lang ist wie a und b zusammengenommen und überführe die entsprechenden Inhalte.
	  #+END_PINNED
	- ```java
	  int[] a  = { 1, 2, 25 };
	  int[] b  = { 9, 18 };
	  
	  // Hier Code schreiben
	  
	  println(c);
	  ```
-
-
-
- ## Zusammenfassung
  collapsed:: true
	- #+BEGIN_TIP
	  Ein Array kann mehrere Dinge vom gleichen Typ speichern. Man kann sich einen Array als Schrank mit durchnummerierten Fächern vorstellen.
	  #+END_TIP
	- Der Datentyp eines Array ist der Typ der enthaltenen Element und dahinter eckige Klammern, z.B. **int[]** für einen Integer-Array oder **String[]** für einen String-Array
	- Array mit 5 Fächern erzeugen: **new int[5]**
	- Elemente sind durchnummeriert, beginnend bei Null. Diese Nummer nennt man Index eines Elements.
	- Zugriff auf ein Element über den Index: foo[3] greift auf das 4. Element zu
	- Länge des Array **foo** mit **foo.length**.
	- Schnelles Erzeugen und Befüllen eines Array: **int[] foo = { 3, 42, -50 };**
-
- ## Grafische Objekte repräsentieren
  collapsed:: true
	- Arrays bieten sich auch an, um mehrere *grafische Objekte* und deren Eigenschaften zu speichern. Wollen wir die x/y-Koordinaten von Rechtecken speichern, verwenden wir zwei Arrays `xpos` und `ypos`. Für uns ist klar: für jeden Index gibt es ein Rechteck und die beiden Elemente, z.B. `xpos[3]` und `ypos[3]` beinhalten die Koordinaten des 4. Rechtecks (nicht die Null vergessen!).
	  
	  
	  ```java
	  // 100 Rechtecke mit je zwei Eigenschaften (xpos, ypos),
	  // die über die Cursortasten verändert werden können
	  
	  int[] xpos = new int[100];
	  int[] ypos = new int[100];
	  
	  void setup() {
	    for (int i = 0; i < xpos.length; i++) {
	      xpos[i] = (int)random(0, width);
	      ypos[i] = (int)random(0, height);
	    }
	  }
	  
	  void draw() {
	    background(0);
	    fill(255);
	    for (int i = 0; i < xpos.length; i++) {
	      rect(xpos[i], ypos[i], 10, 10);
	    }
	  }
	  
	  void keyPressed() {
	    if (keyCode == LEFT) {
	      for (int i = 0; i < xpos.length; i++) {
	        xpos[i]--;
	      }
	    }
	    if (keyCode == RIGHT) {
	      for (int i = 0; i < xpos.length; i++) {
	        xpos[i]++;
	      }
	    }
	    if (key == ' ') {
	      for (int i = 0; i < xpos.length; i++) {
	        xpos[i] = (int)random(0, width);
	        ypos[i] = (int)random(0, height);
	      }
	    }
	  }
	  ```
	- Im oberen Beispiel haben wir 100 Objekte mit je zwei Eigenschaften (xpos und ypos). Jetzt nehmen wir eine dritte Eigenschaft hinzu: die Farbe.
	- ```java
	  // 100 Rechtecke mit je drei Eigenschaften
	  // Die Cursortasten ändern jetzt die jeweilige Graustufe
	  
	  int[] xpos = new int[100];
	  int[] ypos = new int[100];
	  int[] farbe = new int[100];
	  
	  void setup() {
	    for (int i = 0; i < xpos.length; i++) {
	      xpos[i] = (int)random(0, width);
	      ypos[i] = (int)random(0, height);
	      farbe[i] = (int)random(0, 256);
	    }
	  }
	  
	  void draw() {
	    background(0);
	    for (int i = 0; i < xpos.length; i++) {
	      fill(farbe[i]);
	      rect(xpos[i], ypos[i], 10, 10);
	    }
	  }
	  
	  void keyPressed() {
	    if (keyCode == LEFT) {
	      for (int i = 0; i < xpos.length; i++) {
	        farbe[i] -= 5;
	        farbe[i] = constrain(farbe[i], 0, 255);
	      }
	    }
	    if (keyCode == RIGHT) {
	      for (int i = 0; i < xpos.length; i++) {
	        farbe[i] += 5;
	        farbe[i] = constrain(farbe[i], 0, 255);
	      }
	    }
	  }
	  ```
	- Die Farbe muss mit der Funktion `constrain()` im Wertebereich 0..255 gehalten werden.
-
- ## Übung 2
- ### Aufgabe viele Bälle
  collapsed:: true
	- #+BEGIN_PINNED
	  **Aufgabe - viele Bälle**
	  Lasse 10 Bälle in unterschiedlichen Geschwindigkeiten (zufällig) über den Bildschirm fliegen. Gib jedem Ball eine eigene Farbe. Verwende random()in setup(), um die x/y-Koordinaten zufällig zu setzen. Das gleiche gilt für die Geschwindigkeit.
	  #+END_PINNED
- ### Aufgabe viele Bälle Erweiterung
  collapsed:: true
	- #+BEGIN_PINNED
	  **Aufgabe - viele Bälle ERWEITERUNG**
	  Baue ein, dass man jeden einzelnen Ball “ausschalten” kann. Verwende dazu ein Array von booleans. Baue ein, dass man über die Zifferntasten 0,..,9 die entsprechenden Bälle ein- und ausschalten kann.
	  #+END_PINNED
	- #+BEGIN_PINNED
	  **Aufgabe - viele Bälle ERWEITERUNG 2**
	  Führe einen neuen Ball ein, der nicht im Array ist und den du mit den Cursortasten steuern kannst. Sobald diser Spielball einen der anderen Bälle berührt, verschwindet der andere Ball. Überlege dir, wie du die Kollision “auslesen” kannst.
	  #+END_PINNED
	- #+BEGIN_PINNED
	  **Aufgabe - viele Bälle ERWEITERUNG 3**
	  Wenn alle Bälle fort sind, gewinnst du. In diesem Fall setzt du ein “Gewonnen!” auf den Screen.
	  #+END_PINNED
-