## Zeiger
- Variablen, die ein Objekt enthalten, enthalten in Wirklichkeit nur die Adresse des Objekts. Dies hat wichtige Konsequenzen, wie wir gleich sehen werden. Schauen wir uns zwei Variablen mit Objekten an:
- ```
  Ball b1 = new Ball(10,20);
  Ball b2 = new Ball(9,99);
  ```
- Wobei die Klasse `Ball` wie folgt aussieht:
- ```
  class Ball {
   int xpos;
   int ypos;
  - Ball(int x, int y) {
      xpos = x;
      ypos = y;
   }
  }
  ```
- #+BEGIN_NOTE
  Eine Variable, die ein Objekt aufnehmen kann, enthält nicht wirklich das Objekt selbst, sondern nur seine **Adresse im Speicher**. Man sagt auch, die Variable speichert eine **Referenz** oder einen **Zeiger** (engl. pointer) auf das Objekt. Man nennt eine Klasse deshalb auch einen **Referenztyp**.
  #+END_NOTE
- #+BEGIN_NOTE
  Eine **Referenz** kann auf ein konkretes Objekt verweisen oder zeigen. Zeigt sie auf kein Objekt hat sie in Java den Wert **null**. Um eine Referenz zu speichern benötigt man eine **Referenzvariable**. Der Datentyp der Referenzvariablen entspricht dem Datentyp des Objekts (oder einer Oberklasse). Wenn auf ein Objekt nicht mehr referenziert wird, entfernt in Java der Garbage Collector dieses aus dem Speicher. **Objekte haben keine Namen, sondern nur die Referenzen auf Objekte.**
  #+END_NOTE
- Man kann sich das so vorstellen:
- ![oop_zeiger1.png](../assets/oop_zeiger1_1700772613591_0.png)
- Wenn Sie eine Zuweisung zwischen Objektvariablen ausführen, wird lediglich die Adresse eingefüllt. Im folgenden Fall zeigen beide Variablen anschließend auf das identische Objekt:
- ```
  b2 = b1; //Adresse vom b1 wird in b2 gespeichert
  ```
- Dann haben wir folgende Situation:
- ![oop_zeiger2.png](../assets/oop_zeiger2_1700772635730_0.png)
- Weil b1 und b2 auf das selbe Objekt zeigen, kann man sowohl mit b1 als auch mit b2 dieses eine Objekt manipulieren. Wenn Sie schreiben…
- ```
  b1.xpos = 5;
  b2.ypos = 55;
  println(b1.xpos);
  println(b1.ypos);
  ```
- … wird bei beiden Zuweisungen das selbe Objekt manipuliert:
- ![oop_zeiger3.png](../assets/oop_zeiger3_1700772678158_0.png){:height 291, :width 672}
- #+BEGIN_PINNED
  **Aufgabe**
  Die Ausgabe sieht entsprechend aus - Teste es.
  #+END_PINNED
- Denn es gibt nur ein einziges Objekt. Beide Variablen, b1 und b2, zeigen auf dies eine Objekt. Egal, mit welcher Variable Sie dies Objekt manipulieren, so wird immer nur dies eine Objekt manipuliert und der zuletzt gesetzte Wert für eine Instanzvariable gilt.
- Sie können beliebig viele weitere Variablen auf dies Objekt zeigen lassen:
- ```
  Ball b3 = b1;
  Ball b4 = b2;
  Ball b5 = b1;
  Ball b6 = b2;
  ```
- Dabei ist es egal, welche andere Variable Sie als “Zulieferer” für die Adresse verwenden. Es steht überall das gleiche drin.
- ---
- ## Objekte als Parameter (Experte)
- Wichtig ist das bei Funktionsaufrufen. Sehen wir uns zunächst eine Funktion mit einem Parameter an, der einen primitiven Datentyp (int) angehört:
- ```
  void setup() {
   int x = 5;
   foo(x);
   println(x);
  }
  - void foo(int p) {
   p = 10;
  }
  ```
	- **Ausgabe: 5**
- Deine Variable `x` wird durch den Funktionsaufruf nicht geändert, denn Processing kopiert den Wert 5 in die Parametervariable p. Dies hat dann nichts mehr mit x zu tun.
- Die Situation ändert sich, wenn Sie das gleiche mit einem **Objekt** der Klasse `Ball` tun:
- ```
  void setup() {
   Ball b = new Ball(1, 2);
   boo(b);
   println(b.xpos);
  }
  - void boo(Ball ball) {
   ball.xpos = 9;
  }
  ```
- Der Parameter `ball` ist nur eine weitere (lokale) Variable, die nach Funktionsaufruf auf die Adresse des einzigen bislang erzeugten Objekts zeigt:
- ![objektAlsParameter1.png](../assets/objektAlsParameter1_1700772750601_0.png)
- Also bekommen Sie als Ausgabe von `b.xpos`:
	- **Ausgabe: 9**
- #+BEGIN_CAUTION
  Merken musst du dir also, dass bei Übergabe von Objekten an Funktionen diese Objekte geändert werden. Du solltest daher mit Funktionsparametern sehr vorsichtig umgehen und diese nur dann ändern, wenn es notwendig ist.
  #+END_CAUTION
- #+BEGIN_CAUTION
  **Expertenwissen Automatische Speicherfreigabe**
  Der Garbage Collector (Speichermanager) vom .NET Framework verwaltet die Reservierung und Freigabe von Arbeitsspeicher für die Anwendung. Wenn Sie den Operator new zum Erstellen eines Objekts verwenden, reserviert er zur Laufzeit Arbeitsspeicher für das Objekt aus dem verwalteten Heap. Arbeitsspeicher ist jedoch nicht unendlich verfügbar. Möglicherweise muss mithilfe der Garbage Collection Arbeitsspeicher freigegeben werden. Das Optimierungsmodul der Garbage Collection bestimmt den besten Zeitpunkt für die Freigabe anhand der erfolgten Reservierungen. Dabei wird nach Objekten im verwalteten Heap gesucht, die nicht mehr von der Anwendung verwendet werden. Anschließend werden die für das Freigeben des Arbeitsspeichers erforderlichen Operationen ausgeführt.
  #+END_CAUTION
- ---
- ## Objekte als Rückgabewert (Experte)
- Es kommt natürlich auch vor, dass eine Funktion ein Objekt zurückgibt. Zum Beispiel, wenn Sie eine Datenbank durchsuchen. Hier soll das erste Auto zurückgegeben werden, das eine bestimmte Mindest-PS-Zahl hat:
- ```
  Auto findAuto(Auto[] autos, int minPS) {
    for (int i = 0; i < autos.length; i++) {
        if (autos[i].ps > minPS) {
            return autos[i];
        }
    }
    // Problem! Kein return weit und breit...
  }
  ```
- Processing beschwert sich hier. Denn was passiert, wenn ein solches Auto nicht gefunden wird? Processing kommt am Ende der For-Schleife an und findet dort kein return vor. Aber welche Art von Objekt soll denn dann zurückgegeben werden? Was Programmierer in dieser Situation häufig tun, ist, den Wert `null` zurückzugeben:
- ```
  Auto findAuto(Auto[] autos, int minPS) {
    for (int i = 0; i < autos.length; i++) {
        if (autos[i].ps > minPS) {
            return autos[i];
        }
    }
    return null
  }
  ```
- Wer auch immer die Funktion verwendet muss also wissen, dass `null` bedeutet: es wurde nichts gefunden!
- Diese Praxis hat einen unangenehmen Nebeneffekt. Wenn Sie eine Funktion verwenden, die potentiell `null` zurückgibt, müssen Sie aufpassen, wie Sie das zurückgegebene Objekt weiter behandeln:
- ```
  Auto found = findAuto(meineAutos, 300);
  found.printSteckbrief(); // Vorsicht! Könnte null sein.
  ```
- Es könnte sein, dass in der Variablen `found` der Wert `null` steckt. Sie müssen also, bevor Sie die Methode `printSteckbrief()` aufrufen, sicherstellen, dass `found` nicht `null` enthält:
- ```
  Auto found = findAuto(meineAutos, 300);
  if (found != null) {}
    found.printSteckbrief();
  }
  ```
- Solche Abfragen siehst du sehr häufig, wenn mit Objekten hantiert wird!