- Objekte können so viel mehr als einfache Variablen! Sie speichern mehrere Informationen in **Eigenschaften** und stellen sogar eigene Funktionen bereit - sogenannte **Methoden**.
- Es gibt viele verschiedene Typen von Objekten, später werden Sie diese selbst programmieren. Einen bestimmten Typ von Objekten nennt man auch eine Klasse (engl. class). Die **Klasse** ist sozusagen der Bauplan einer Sorte von Objekten. Für Vektoren gibt es die Klasse PVector, für Zeichenketten die Klasse **String**, für Digitalbilder die Klasse **PImage** usw.
- Wir schauen uns jetzt die Klasse `PVector` an.
-
-
- ## Objekte erzeugen
- Ein Vektor hat eine x- und eine y-Komponente, das sind die Eigenschaften eines jeden Objekts. Wir wollen einen neuen Vektor mit den Werten (4, 2) erstellen. Dazu definieren wir zunächst eine neue Variable `vec` vom Typ `PVector`:
  
  ```java
  PVector vec; //kann einen Vektor speichern
  ```
- Du siehst: das funktioniert genauso wie bei einer int-Variablen.
- Subtiler Unterschied: **PVector wird groß geschrieben**. Die Großschreibung weist darauf hin, dass wir es nicht mit einem primitiven Datentypen, sondern mit einem **Objekttypen** zu tun haben.
- Unsere Variable `vec` ist zunächst “leer”. Im Gegensatz zu einer int-Variable, die im Zweifelsfall 0 ist, enthält unsere Variable das Stichwort `null`, das bedeutet, dass die Variable derzeit “auf kein Objekt zeigt”. Wir haben ja auch noch gar kein Objekt hergestellt. Also sieht die Situation wie folgt aus:
- ![pvector_erzeugen1.PNG](../assets/pvector_erzeugen1_1700774995643_0.PNG)
- Jetzt erzeugen wir das neue Vektorobjekt, das in `vec` gespeichert wird. Dazu schreiben wir das Schlüsselwort `new`, dann den Namen der Klasse und dann einige Parameter in Klammern. Die Art und Anzahl dieser Parameter kann variieren, bei `PVector` haben wir zwei float-Parameter für x und y:
- ```java
  vec = new PVector(4, 2); // neuer Vektor mit x=4 und y=2
  ```
- Hier passieren zwei Dinge. Erstens wird mit `new PVector(4, 2)` ein neues Objekt im Speicher angelegt.
- ![pvector_erzeugen2.PNG](../assets/pvector_erzeugen2_1700775026215_0.PNG)
- Im nächsten Schritt wird in der Variablen vec eine Referenz auf das Objekt angelegt. Das kann man sich als Pfeil oder Zeiger in den Speicher vorstellen.
- ![pvector_erzeugen3.PNG](../assets/pvector_erzeugen3_1700775038967_0.PNG)
- Natürlich kann man Variablendeklaration und Zuweisung wie gewohnt auf einer Zeile durchführen:
  
  ```java
  PVector vec = new PVector(4, 2);
  ```
- Wenn Sie den Nullvektor (0, 0) haben möchten, können Sie auch die Parameter weglassen, nur die Klammern müssen Sie unbedingt hinschreiben.
- ```java
  PVector vec = new PVector(); // Nullvektor
  ```
-
-
- ## Eigenschaft
- Wir haben jetzt eine neue Variable `vec` vom Typ `PVector`. Wie arbeiten wir jetzt mit dieser Variablen?
- Wir können einerseits Eigenschaften auslesen (hier wären das x und y), indem wir den Variablennamen, dann einen Punkt, dann die Eigenschaft hinschreiben:
  
  ```java
  println(vec.x);
  println(vec.y);
  ```
- Dies nennt man auch die **Punktnotation** und erlaubt es uns, zwischen den Eigenschaften verschiedener Objekten zu unterscheiden. Definieren wir ein zweites Vektorobjekt in `vec2` …
- Schematisch kann man sich das so vorstellen:
  ![pvector_erzeugen4.PNG](../assets/pvector_erzeugen4_1700775105472_0.PNG)
-
- …dann möchten Sie natürlich zwischen den x/y-Koordination von vec und vec2 unterscheiden:
  
  ```java
  println(vec.x);
  println(vec.y);
  println(vec2.x);
  println(vec2.y);
  ```
- Sie können sich die Objekte jeweils in ihrer Gesamtheit mit println() betrachten:
  
  ```java
  PVector vec = new PVector(4, 2);
  PVector vec2 = new PVector(10, -30);
  println(vec);
  println(vec2)
  ```
- #+BEGIN_PINNED
  Betrachte die entsprechende Ausgabe.
  #+END_PINNED
- Was du eigentlich siehst, ist ein dreidimensionaler Vektor, also die Eigenschaft x,y und z hat. Ignoriere einfache z ;-)
- Du kannst auch Eigenschaften durch Zuweisung ändern:
  
  ```java
  PVector v = new PVector(10, 20);
  
  println(v);
  
  v.x = 200;
  v.y = 1;
  
  println(v);
  ```
-
-
- ## Methoden
- Objekte können nicht nur mehr Informationen speichern als Variablen, sie bieten darüber hinaus auch Aktionen an, die etwas “tun”, also etwas berechnen oder bewegen.
- Ein Vektor kann z.B. seine Länge berechnen:
  
  ```java
  float l = vec.mag();
  ```
- Die Methode `mag()` (für engl. magnitude) berechnet die Länge des Vektors und gibt diese zurück. **Beachten Sie die Punktnotation und die Klammern!**
- Um die Eigenschaften x und y des Vektors zu **ändern**, können Sie die Methode `set()` verwenden:
  
  ```java
  PVector vec = new PVector();
  vec.set(0, 50); // setzt x=0 y=50
  ```
- Die Methode `mult()` multipliziert den gesamten Vektor (also sowohl x als auch y) mit einer Zahl. **Diese Methode ändert also die Eigenschaften des Objekts!**
- ```java
  PVector vec = new PVector(2, 3);
  
  println("vorher:" + vec);
  vec.mult(10);
  println("nachher: " + vec);
  ```
- Die Methode `add()` addiert einen anderen Vektor zu dem eigenen Vektor hinzu. Im folgenden Code wird `vec` also wieder verändert:
  
  ```java
  PVector vec = new PVector(1, 1);
  PVector vec2 = new PVector(10, 10);
  
  vec.add(vec2);
  
  println(vec);
  ```
- Ähnlich funktionieren die Methoden `sub()` (Subtraktion), `dot()` (Skalar- oder Punktprodukt) und `cross()` (Kreuz- oder Vektorprodukt). Jedesmal wird der Vektor, auf dem Sie die Methode aufrufen, verändert.
- Sicherlich noch von Interesse ist die Methode `dist()`, welche die Distanz zwischen zwei Punkten misst (hier wird ein Vektor als Punkt x, y im Raum interpretiert):
  
  ```java
  PVector v1 = new PVector(1,1);
  PVector v2 = new PVector(5,1);
  float d = v1.dist(v2);
  
  println(d);
  ```
-
-
- ## Identität und Gleichheit
- Nehmen wir an, Sie haben zwei Vektoren v und w:
  
  ```java
  PVector v = new PVector(5, 0);
  PVector w = new PVector(5, 0);
  println(v == w);
  ```
- Es erscheint in der Ausgabe **false**.
- Offensichtlich findet Processing **nicht**, dass v und w gleich sind. Warum ist das so? Weil v und w zwei unterschiedliche Objekte sind, wie man im Bild des Speichers oben klar sieht.
- Das == prüft also, ob zwei Variablen das selbe Objekt beinhalten. Man kann auch sagen: Das == prüft die **Identität**.
- Möchten wir prüfen, ob zwei Vektoren die gleichen **Werte** haben, können wir die PVector-Methode `equals()`verwenden:
  
  ```java
  println(v.equals(w));
  ```
- Ausgabe: **true**
- Du musst bei Objekten also immer unterscheiden zwischen
	- **Identität**: Zwei Variablen beinhalten das selbe Objekt. Man sagt auch: zwei Variablen “zeigen” auf das selbe Objekt. Dies wird mit == geprüft.
	- **Gleichheit**: Zwei Variablen zeigen auf Objekte, die die gleichen Eigenschaften haben. Bei Vektoren heißt das: zwei Vektorobjekte haben die gleichen x/y-Werte. Dies wird mit der Methode equals() geprüft.
-
- ## Übung
- #+BEGIN_PINNED
  Erzeuge einen Vektor v1=(5,5) und v2=(3,2) und berechne sowohl v1+v2 und v1-v2. Gib die Ergebnisse in der Konsole aus.
  #+END_PINNED
- #+BEGIN_PINNED
  Lasse einen Ball von links nach rechts über den Bildschirm fliegen. Verwende dazu einen Vektor. Der Ball soll in der Mitte des Bildschirms starten. Wenn der Ball rechts austritt, soll er links wieder eintreten.
  #+END_PINNED
- #+BEGIN_PINNED
  Schreibe ein Programm, wo ein Ball durch den 2D-Raum fliegt und von den Wänden abprallt. Verwende einen Vektor für die Position und einen Vektor für die Geschwindigkeit.
  Wähle als Startposition die Mitte des Bildschirms. Die Startgeschwindigkeit sollte im Bereich -3 bis 3 liegen, sowohl in x- als auch in y-Richtung.
  #+END_PINNED
- #+BEGIN_PINNED
  Nutzen Sie die Methode dist(), um zu testen, ob der Ball aus den obigen Codeschnipseln mit der Maus kollidiert.
  #+END_PINNED