- In einer Klassenhirarchie sollen Unterklassen in bestimmten Fällen eine Methode der Oberklasse ersetzen. Stelle dir vor, du programmierst ein Computerspiel und die meisten Spielobjekte bewegen sich mit Hilfe eines Speed-Vektors in der Methode `move()`. Eine bestimmte Klasse von Objekten soll sich aber zufällig bewegen. Dann musst du dort die Methode `move()` neu definieren. Man nennt dies auch *überschreiben* (override).
- Wir schauen uns das anhand einer einfachen Print-Anweisung an:
- ```java
  class GameObject {
    void saySomething() {
      println("Hello!");
    }
  }
  
  class Rocket extends GameObject {
  
    // überschreibt Methode von GameObject!
    void saySomething() {
      println("Wooosh!");
    }
  }
  
  class Alien extends GameObject {
  }
  ```
- Wenn du die Methode von drei verschiedenen Objekten aufrufst, bekommst du:
- ```java
  void setup() {
  
    GameObject g = new GameObject();
    g.saySomething();
  
    // im Folgenden nimmt Processing die Methode aus Rocket,
    // obwohl g vom Typ GameObject ist!
  
    g = new Rocket();
    g.saySomething();
  
    g = new Alien();
    g.saySomething();
  }
  ```
- **Ausgabe**
  Hello!
  Woosh!
  Hello!
- Was ist passiert? Bei dem ersten `saySomething()` wird die Methode von `GameObject` genommen - alles normal hier. Beim zweiten `saySomething()` enthält `g` ein Objekt vom Typ `Rocket` und Processing nimmt auch die Methode `saySomething()` von `Rocket`. Denn diese Methode hat die von GameObject überschrieben. Im dritten Fall sucht Processing zuerst in `Alien`, findet dort keine Methode `saySomething()` und nimmt dann die der Oberklasse GameObject.
- Du siehst, dass Processing/Java den **dynamischen Typ** nimmt, um die Methode auszuwählen. Das ist auch sehr sinnvoll, denn du willst natürlich immer den spezialisiertesten Code ausführen. Du kannst dir also auch merken: Wenn Java eine Methode in einer Klassenhierarchie auswählen soll, wählt es immer die spezialisierteste!
- #+BEGIN_NOTE
  Eine **Methode** heißt **polymorph**, wenn gleichnamige Methoden unterschiedliche Implementierungen haben.
  Polymorphie wird durch Überschreiben von geerbten Methoden bewirkt.
  Durch die sogenannte **späte** (auch **dynamische**) **Bindung** wird erst zur Laufzeit des Programms in Abhängigkeit vom Typ des Obkjektes entschieden, welche dieser Methoden durch ein und dieselbe Anweisung zur Ausführung kommt.
  #+END_NOTE
- ## Animierte Objekte
- Du erinnerst dich hoffentlich an unser Beispiel mit den animierten Bällen und Quadraten.
- ![oop_polymorphie1.png](../assets/oop_polymorphie1_1700773426383_0.png)
  collapsed:: true
	- Grafik von Michael Kipp
- Unser Ziel ist es, viele Objekte mit einer Liste zu bewegen:
- ```java
  ArrayList<Mover> things = new ArrayList<Mover>();
  
  for (Mover m: things) {
    m.update();
    m.render(); // Fehler: Mover hat kein render
  }
  ```
- Problem ist: Die Klasse `Mover` hat keine Methode `render()`. Daher kann Java das render in der Schleife nicht aufrufen. Die Lösung ist eine “Dummy-Methode” in Mover einzuführen
- ```javascript
  class Mover {
    PVector location;
    PVector speed;
  
    // Code zur Übersichtlichkeit ausgelassen
  
    void render() {
    }
  }
  ```
- Unser Klassendiagramm sieht jetzt so aus
- ![oop_polymorphie2.png](../assets/oop_polymorphie2_1700773470932_0.png)
  collapsed:: true
	- Grafik von Michael Kipp
- Wir können also jetzt die Methode `render()` in unserer Schleife aufrufen, da Mover ein render() besitzt:
- ```java
  ArrayList<Mover> things = new ArrayList<Mover>();
  
  for (Mover m: things) {
    m.update();
    m.render();
  }
  ```
- Da wir wissen, dass jedes Element auf jeden Fall entweder vom Typ Ball oder vom Typ Quad ist, wird immer eine der zwei speziellen render-Methoden aufgerufen. Die Methode von Mover wird nie verwendet!
- **Aber:** Unsere Lösung ist noch nicht optimal! Eine “Dummy-Methode” könnte einen nicht gut eingeweihten Programmierer dazu verführen, Code dort hineinzuschreiben, der aber nie ausgeführt wird. Zweites Problem ist die Klasse Mover selbst. Derselbe Programmierer könnte auf die Idee kommen, eine Instanz von Mover anzulegen. Das aber ist nicht sinnvoll, da Mover aus rein “organisatorischen” Gründen angelegt wurde, aber nicht dazu da ist, um Mover-Objekte zu erzeugen. Beide Probleme werden im Kapitel “Abstract” gelöst.
- #+BEGIN_WARNING
  **Zusammenfassung**
  Eine Unterklasse kann eine Methode der Oberklasse mit der gleichen Signatur neu definieren. Dies nennt man **Überschreiben**. (Im Englischen nennt man das method overriding und NICHT overwriting - to override bedeutet “außer Kraft setzen” oder “etwas überordnen”).
  
  Wird eine Methode mit einer Variablen x aufgerufen, verwendet Java immer den **dynamischen Typ** von x. Java wählt also immer die spezifischste Methode aus.
  
  **Polymorphie** (Vielgestaltigkeit) bedeutet in diesem Zusammenhang, dass der Aufruf x.render() unterschiedliche Ausprägungen annehmen kann, je nachdem, von welchem (dynamischen) Typ x ist.
  #+END_WARNING
-
-
- ## Übung
- Hier gehts zu den [[Aufgaben Polymorphie]].
-