- Bisher war da noch nichts Neues. Doch was ist aufgefallen…
- ## Probleme und Code-Duplizierung
- Unser Programm hat einige unschöne Aspekte:
- Die Klassen `Movie` und `Song` sehen fast gleich aus. Das bedeutet, dass Code doppelt vorhanden ist. Man nennt das Code-Duplizierung. Das gilt als schlechte Programmierpraxis, weil ich bei Änderungen in einem Code (z.B. einen Fehler beheben oder den Code schneller/eleganter gestalten) auch den anderen ändern müsste.
- Wir müssen die Medien in zwei Listen verwalten, besser wäre alles in einer Datenstruktur.
-
- #+BEGIN_WARNING
  Ein wichtiges Designprinzip vom Erstellen von Klassen ist also die Vermeidung von Code-Duplizierung, im Englischen spricht man auch von DRY: Don’t Repeat Yourself. Dies wird durch Vererbung vermieden.
  #+END_WARNING
- ## Aufbau von Vererbung
- Wichtig dabei sind zwei Schlüsselbegriffe:
	- `extends` und
	- `super()`
- ![vererbung-medium.png](../assets/vererbung-medium_1700772884242_0.png)
	- Grafik von Michael Kipp
- #+BEGIN_NOTE
  Medium ist eine sogenannte Oberklasse von Movie und Song. Die sind jeweils eine Unterklasse von Medium. Attribute oder Methoden die gleichwertig verwendet werden können, sollte man folglich in der Oberklasse definieren. Da Movies als auch Songs einen Titel und einen Dateinamen besitzen, werden diese in der Oberklasse initialisiert. Diesen Konstruktorverweis muss man mittels super() aufrufen. Das Ableiten einer Klasse von einer Oberklasse wird mittels extends aufgezeigt.
  #+END_NOTE
- Sobald du eine Klasse A schreibst, erzeugst du auch einen neuen Datentypen A, denn du kannst natürlich dann Variablen herstellen, die Instanzen von A speichern sollen.
  Wenn Klasse B Unterklasse von A ist, dann ist der Datentyp B automatisch Subtyp von A. Das bedeutet, dass eine Variable von Typ A auch Objekte von Typ B speichern kann.
- #+BEGIN_NOTE
  **Generalisierung**
  Aus bestehenden Klassen wird eine allgemeine Superklasse gebildet, welche die Schnittmenge der Attribute und Methoden verwaltet.
  #+END_NOTE
- #+BEGIN_NOTE
  **Spezialisierung**
  Durch Hinzufügen von speziellen Attributen, Methoden oder Beziehungen kann aus einer allgemeinen Superklasse eine spezielle Subklasse abgeleitet werden.
  #+END_NOTE
- #+BEGIN_WARNING
  **Zugriffsberechtigung**
  Mit der Zugriffsberechtigung protected wird sichergestellt, dass alle Subklassen Zugriff auf die entsprechenden Attribute und Methoden der Superklasse haben.
  #+END_WARNING
- Unsere Klasse Medium erhält einen eigenen Konstruktor:
- ```java
  class Medium {
   String title;
   String file;
  
   Medium(String aTitle, String aFile) {
    title = aTitle;
    file = aFile;
   }
  }
  ```
- Um Code-Duplizierung zu vermeiden, kannst du im Konstruktor einer Klasse den Konstruktor der Oberklasse aufrufen. In unserem Beispiel werden die Variablen title und file im Konstruktor von Medium initialisiert. Wir wollen diesen Code nicht im Konstruktor von Movie duplizieren.
- Also rufen wir im Konstruktor von Movie den Konstruktor der Oberklasse mit `super()` auf. super() hat immer die gleiche Anzahl von Parametern wie der Konstruktor (bzw. einer der Konstruktoren) der Oberklasse. In diesem Fall sind das zwei Parameter:
- ```java
  class Movie extends Medium {
   String director;
   Movie(String aTitle, String aDirector, String aFile) {
    super(aTitle, aFile);
    director = aDirector;
   }
   String toString() {
    return "MOVIE \"" + title + "\" von " + director + ", zu finden unter: " + file;
   }
  }
  ```
- Wichtig: super() muss immer die erste Anweisung im Konstruktor sein. Es gibt übrigens noch eine weitere Verwendung von super, um Methoden der Oberklasse aufzurufen. Letzteres wird aber ohne Klammern geschrieben und behandeln wir später. Nicht verwechseln!
- Analog für die Klasse Song:
- ```java
  class Song extends Medium {
   String artist;
   Song(String aTitle, String aArtist, String aFile) {
    super(aTitle, aFile);
    artist = aArtist;
   }
   String toString() {
    return "SONG \"" + title + "\" von " + artist + ", zu finden unter: " + file;
   }
  }
  ```
- Wir passen den Teil an, wo wir die Listen erzeugen:
- Jetzt können wir alle Objekte in einer Liste vom Typ `Medium` ablegen, da `Movie` und `Song` Subtypen von Medium sind. Die Verwaltung wird so wesentlich übersichtlicher:
- ```java
  void setup() {
  // Nur noch eine Liste für beide Medientypen
   ArrayList<Medium> media = new ArrayList<Medium>();
   media.add(new Movie("Alien", "Ridley Scott", "alien.mov"));
   media.add(new Movie("Der Pate", "Francis Ford Coppola", "pate.mov"));
   media.add(new Movie("Gravity", "Alfonso Cuarón", "gravity.avi"));
   media.add(new Song("OMG!", "Marteria", "omg.wav"));
   media.add(new Song("Happy", "Pharrell Williams", "happy.wav"));
   // Nur noch eine For-Schleife zur Ausgabe
   for(int i = 0 ; i < media.size() ; i++){
    println(media.get(i).toString());
   }
  }
  ```
-
-
- ### Weitere Informationen
	- Hat deine Klasse eine Oberklasse, dann wird in allen Konstruktoren automatisch super() aufgerufen, sobald eine Instanz erzeugt wird - sofern nicht ein eigenes super() vom Programmierer eingefügt wurde.
	- Das Verhalten der Vererbung ist transitiv. Wenn A Oberklasse von B und B Oberklasse von C ist auch A Oberklasse von C.
- ---
- **Nerd-Wissen**
- Es gibt Fälle, wo das If-Else etwas sperrig ist. Nehmen wir an, dein Programm soll ausgeben, ob jemand jung (< 40 Jahre) oder alt (>= 40 Jahre) ist:
- ```java
  String altersklasse(int alter) {
   if (alter < 40) {
    return "jung";
   } else {
    return "alt";
   }
  }
  ```
- Der sogenannte ternäre Operator erlaubt es dir, diese drei Bestandteile (Bedingung, Fall 1, Fall 2) sehr kompakt in folgender Form zu schreiben:
- ```java
  String altersklasse(int alter) {
   return alter < 40 ? "jung" : "alt";
  }
  ```
- Der ternäre Operator wird auch häufig verwendet, um Strings zusammenzusetzen. Dann musst du den ganzen Ausdruck einklammern:
- ```java
  int alter = 50;
  println("Sie sind " + (alter < 40 ? "jung" : "alt"));
  ```
-
- ## Alternative Erklärung
- Vererbung beschreibt eine neue Beziehung (auch ist-ein-Beziehung genannt) zwischen einer **Superklasse** (Oberklasse, Basisklasse) und einer **Subklasse** (Unterklasse, abgeleitete Klasse). Eine Subklasse erbt (erhält) von der Superklasse alle Attribute und Methoden bis auf den Konstruktor.
- ### Objekterzeugung in der Subklasse
- Der Konstruktor der Superklasse kann mittels `super(...)` aufgerufen werden. Mit den restlichen Anweisungen des Konstruktors werden die speziellen Attribute der Subklasse initialisiert.
- Der Konstruktor der Superklasse muss die erste Anweisung im Konstruktor der Subklasse sein. Fehlt diese Anweisung, so wird automatisch der parameterlose Konstruktor der Superklasse aufgerufen.
- Im Beispiel:
- ```java
  //Konstruktor der Superklasse
  public Schueler(String name, String klasse) {
    this.name = name;
    this.klasse = klasse;
   // Konstruktor der Subklasse
  public Klassensprecher(String name, String klasse,int amtszeit) {
    super(name,klasse);
    this.amtszeit = amtszeit;
  }
  ```
- #+BEGIN_TIP
  Vererbung kann mehrstufig eingesetzt werden. Auf diese Weise entsteht eine **Vererbungshierarchie**.
  #+END_TIP
-
- ## Übung
- Hier geht es zu den [[Aufgaben Vererbung]].